/*
 * Renderer 9. The MIT License.
 * Copyright (c) 2022 rlkraft@pnw.edu
 * See LICENSE for details.
*/

import renderer.scene.*;
import renderer.scene.primitives.*;
import renderer.scene.util.ModelShading;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the letter {@link P} around the tip of its front,
   with the tip located at (-1,0).
<p>
   See the animation <a href="../Example_13.gif">Example_13.gif</a>.
<p>
   <a href="../Example_13.gif"><img src="../Example_13.gif" alt="Example_13.gif"></a>
*/
public class Example_13
{
   public static void main(String[] args)
   {
      // Since this is a 2D example, we can use an orthographic camera.
      final double right  = 2.0;
      final double left   = -right;
      final double top    = right;
      final double bottom = -top;
      final Scene scene = new Scene("Example_13",
                                 Camera.projOrtho(left, right, bottom, top));

      // Create a FrameBuffer to render our scene into.
      final int width  = 512;
      final int height = 512;
      final FrameBuffer fb = new FrameBuffer(width, height, Color.black);

      // Create a set of x and y axes.
      final Model axes = new Axes2D(left, right, bottom, top, 8, 8);
      ModelShading.setColor(axes, Color.red);
      final Position axes_p = new Position(axes);
      scene.addPosition(axes_p);

      // Create a Model of the letter P.
      final Model modelP = new P();
      ModelShading.setColor(modelP, new Color(50, 150, 170));
      final Position modelP_p = new Position(modelP);
      scene.addPosition(modelP_p);


      // Rotate the letter P around the tip of its front, with the tip located at (-1,0).
      for (int i = 0; i < 72; ++i)
      {
         // Transform the model.
         modelP_p.transform( Matrix.translate(-1.0,  0, 0)
                     .times( Matrix.rotateZ(5*i) )
                     .times( Matrix.translate(-1.0, -0.7, 0) ) );

         // Render again.
         fb.clearFB();
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_13_Frame%03d.ppm", i));
      }
   }
}
