/*
 * Renderer 9. The MIT License.
 * Copyright (c) 2022 rlkraft@pnw.edu
 * See LICENSE for details.
*/

import renderer.scene.*;
import renderer.scene.primitives.*;
import renderer.scene.util.ModelShading;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the letter {@link P} around the top of the {@link P}
   while scaling it twice as large for each rotation.
<p>
   The top of {@link P} should always be equal to the height of {@link P}.
<p>
   See the animation <a href="../Example_33.gif">Example_33.gif</a>.
<p>
   <a href="../Example_33.gif"><img src="../Example_33.gif" alt="Example_33.gif"></a>
*/
public class Example_33
{
   public static void main(String[] args)
   {
      // Set up an orthographic camera.
      final double right  = 4.0;
      final double left   = -right;
      final double top    =  right;
      final double bottom = -top;
      final Scene scene = new Scene("Example_33",
                             Camera.projOrtho(left, right, bottom, top));

      // Create a FrameBuffer to render our scene into.
      final int width  = 512;
      final int height = 512;
      final FrameBuffer fb = new FrameBuffer(width, height, Color.black);

      // Create a set of x and y axes.
      final Model axes = new Axes2D(left, right, bottom, top, 16, 16);
      ModelShading.setColor(axes, Color.red);
      final Position axes_p = new Position(axes);
      scene.addPosition(axes_p);

      // Create a Model of the letter P.
      final P modelP = new P();
      ModelShading.setColor(modelP, Color.blue);
      final Position modelP_p = new Position(modelP);
      scene.addPosition(modelP_p);


      // Rotate the letter P around the top of the P
      // while scaling it twice as large for each rotation.
      // The top of P should always be equal to the height of P.
      for (int i = 0; i <= 120; ++i)
      {
         final double scale = Math.pow(2, i/60.0);

         // scale, translate, rotate, translate
         modelP_p.transform( Matrix.scale(scale, scale, 1)
                     .times( Matrix.translate(0, 1, 0) )
                     .times( Matrix.rotateZ(i*6) )
                     .times( Matrix.translate(0, -1, 0) ) );
/*
         // translate, rotate, translate, scale
         modelP_p.transform( Matrix.translate(0, scale, 0)
                     .times( Matrix.rotateZ(i*6) )
                     .times( Matrix.translate(0, -scale, 0) )
                     .times( Matrix.scale(scale, scale, 1) ) );
*/
/*
         // Try to do this animation with the transformations
         // in the following order (from top to bottom),
         // translate, scale, rotate, translate.




*/
         // Render again.
         fb.clearFB();
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_33_Frame%03d.ppm", i));
      }
   }
}
