/*
 * Renderer 9. The MIT License.
 * Copyright (c) 2022 rlkraft@pnw.edu
 * See LICENSE for details.
*/

import renderer.scene.*;
import renderer.scene.primitives.*;
import renderer.scene.util.ModelShading;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Scale the letter {@link P} to four times its original size
   while keeping the top of the {@link P} at the line y = 1 and
   the right-hand edge of the {@link P} at the line x = 1.
<p>
   See the animation <a href="../Example_34.gif">Example_34.gif</a>.
<p>
   <a href="../Example_34.gif"><img src="../Example_34.gif" alt="Example_34.gif"></a>
*/
public class Example_34
{
   public static void main(String[] args)
   {
      // Set up an orthographic camera.
      final double right  = 4.0;
      final double left   = -right;
      final double top    =  right;
      final double bottom = -top;
      final Scene scene = new Scene("Example_34",
                             Camera.projOrtho(left, right, bottom, top));

      // Create a FrameBuffer to render our scene into.
      final int width  = 512;
      final int height = 512;
      final FrameBuffer fb = new FrameBuffer(width, height, Color.black);

      // Create a set of x and y axes.
      final Model axes = new Axes2D(left, right, bottom, top, 16, 16);
      ModelShading.setColor(axes, Color.red);
      final Position axes_p = new Position(axes);
      scene.addPosition(axes_p);

      // Create a Model of the letter P.
      final P modelP = new P();
      ModelShading.setColor(modelP, Color.blue);
      final Position modelP_p = new Position(modelP);
      scene.addPosition(modelP_p);


      // Scale the letter P to four times its original size while keeping
      // the top of the P at the line y = 1 and the right-hand edge of
      // the P at the line x = 1.
      for (int i = 0; i <= 120; i++)
      {
         final double scale = 1.0 + i/40.0;
         // translate then scale
         modelP_p.transform( Matrix.translate(1 - scale,
                                              1 - scale,
                                              0)
                     .times( Matrix.scale(scale, scale, 1) ) );

         // Render again.
         fb.clearFB();
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_34_Frame%03da.ppm", i));
      }


      for (int i = 0; i <= 120; i++)
      {
         final double scale = 1.0 + i/40.0;
         // scale then translate
         modelP_p.transform( Matrix.scale(scale, scale, 1)
                     .times( Matrix.translate(1/scale - 1,
                                              1/scale - 1,
                                              0) ) );
         // Render again.
         fb.clearFB(Color.darkGray.darker());
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_34b_Frame%03db.ppm", i));
      }
   }
}
