/*
 * Renderer 9. The MIT License.
 * Copyright (c) 2022 rlkraft@pnw.edu
 * See LICENSE for details.
*/

import renderer.scene.*;
import renderer.scene.primitives.*;
import renderer.scene.util.ModelShading;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Scale the letter {@link P} larger as its left edge
   forms a 45 degree diagonal from the positive
   y-axis to the positive x-axis.
<p>
   See the animation <a href="../Example_35.gif">Example_35.gif</a>.
<p>
   <a href="../Example_35.gif"><img src="../Example_35.gif" alt="Example_35.gif"></a>
*/
public class Example_35
{
   public static void main(String[] args)
   {
      // Set up an orthographic camera.
      final double right  = 4.0;
      final double left   = -right;
      final double top    =  right;
      final double bottom = -top;
      final Scene scene = new Scene("Example_35",
                             Camera.projOrtho(left, right, bottom, top));

      // Create a FrameBuffer to render our scene into.
      final int width  = 512;
      final int height = 512;
      final FrameBuffer fb = new FrameBuffer(width, height, Color.black);

      // Create a set of x and y axes.
      final Model axes = new Axes2D(left, right, bottom, top, 16, 16);
      ModelShading.setColor(axes, Color.red);
      final Position axes_p = new Position(axes);
      scene.addPosition(axes_p);

      // Create a Model of the letter P.
      final P modelP = new P();
      ModelShading.setColor(modelP, Color.blue);
      final Position modelP_p = new Position(modelP);
      scene.addPosition(modelP_p);


      // Scale the letter P larger as its left edge
      // forms a 45 degree diagonal from the positive
      // y-axis to the positive x-axis.
      for (int i = 2; i <= 64; i++)
      {
         final double scale = Math.pow(i/2.0, 0.5);
         modelP_p.transform( Matrix.scale(scale, scale, 1)
                     .times( Matrix.translate(1/Math.pow(2, 0.5), 0, 0) )
                     .times( Matrix.rotateY(180) )
                     .times( Matrix.rotate(-45, 0,0,1) ) );

         // Render again.
         fb.clearFB();
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_35_Frame%03d.ppm", i));
      }


      for (int i = 2; i <= 64; i++)
      {
         final double scale = Math.pow(i/2.0, 0.5);
         modelP_p.transform( Matrix.rotateZ( 45)
                     .times( Matrix.rotateY(180) )
                     .times( Matrix.scale(scale, scale, 1) )
                     .times( Matrix.translate(-0.5, -0.5, 0) ) );

         // Render again.
         fb.clearFB(Color.darkGray.darker());
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_35b_Frame%03d.ppm", i));
      }
   }
}
