/*
 * Renderer 9. The MIT License.
 * Copyright (c) 2022 rlkraft@pnw.edu
 * See LICENSE for details.
*/

import renderer.scene.*;
import renderer.scene.primitives.*;
import renderer.scene.util.ModelShading;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the letter {@link P} down the x-axis as it grows larger.
<p>
   See the animation <a href="../Example_36_puzzle.gif">Example_36_puzzle.gif</a>.
<p>
   <a href="../Example_36_puzzle.gif"><img src="../Example_36_puzzle.gif" alt="Example_36.gif"></a>
*/
public class Example_36
{
   public static void main(String[] args)
   {
      // Set up an orthographic camera.
      final double right  = 11.0;
      final double left   = -right;
      final double top    =  right;
      final double bottom = -top;
      final Scene scene = new Scene("Example_36",
                             Camera.projOrtho(left, right, bottom, top));

      // Create a FrameBuffer to render our scene into.
      final int width  = 512;
      final int height = 512;
      final FrameBuffer fb = new FrameBuffer(width, height, Color.black);

      // Create a set of x and y axes.
      final Model axes = new Axes2D(left, right, bottom, top, 22, 22);
      ModelShading.setColor(axes, Color.red);
      final Position axes_p = new Position(axes);
      scene.addPosition(axes_p);

      // Create a Model of the letter P.
      final P modelP = new P();
      ModelShading.setColor(modelP, Color.blue);
      final Position modelP_p = new Position(modelP);
      scene.addPosition(modelP_p);


      // Rotate the letter P down the x-axis as it grows larger.

      // Do the first 90 degrees in steps of 3 degrees.
      for (int i = 0; i <= 30; i++)
      {






         // Render again.
         fb.clearFB();
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_36_Frame%03d.ppm", i));
      }


      // Do the next 180 degrees in steps of 3 degrees.
      for (int i = 1; i <= 60; i++)
      {







         // Render again.
         fb.clearFB();
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_36_Frame%03d.ppm", 30+i));
      }


      // Do the last 90 degrees in steps of 3 degrees.
      for (int i = 1; i <= 30; i++)
      {







         // Render again.
         fb.clearFB();
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_36_Frame%03d.ppm", 90+i));
      }
   }
}
